# -*- coding: utf-8 -*-
from . import casper_pb2 as casper


# Note, there is also casper.StateQuery.KeyVariant.KEY_VARIANT_UNSPECIFIED,
# but it doesn't seem to have an official string representation
# ("key_variant_unspecified"? "unspecified"?) and is not used by the client.
STATE_QUERY_KEY_VARIANT = {
    "hash": casper.StateQuery.KeyVariant.HASH,
    "uref": casper.StateQuery.KeyVariant.UREF,
    "address": casper.StateQuery.KeyVariant.ADDRESS,
    "local": casper.StateQuery.KeyVariant.LOCAL,
}


def key_variant(key_type):
    """
    Returns query-state key variant.
    """
    variant = STATE_QUERY_KEY_VARIANT.get(key_type.lower(), None)
    if variant is None:
        raise ValueError(f"{key_type} is not a known query-state key type")
    return variant



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: gossiping.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

import empty_pb2
import consensus_pb2
import node_pb2
import gossiping_pb2


class GossipServiceBase(abc.ABC):

    @abc.abstractmethod
    async def NewBlocks(self, stream: 'grpclib.server.Stream[gossiping_pb2.NewBlocksRequest, gossiping_pb2.NewBlocksResponse]') -> None:
        pass

    @abc.abstractmethod
    async def NewDeploys(self, stream: 'grpclib.server.Stream[gossiping_pb2.NewDeploysRequest, gossiping_pb2.NewDeploysResponse]') -> None:
        pass

    @abc.abstractmethod
    async def StreamAncestorBlockSummaries(self, stream: 'grpclib.server.Stream[gossiping_pb2.StreamAncestorBlockSummariesRequest, consensus_pb2.BlockSummary]') -> None:
        pass

    @abc.abstractmethod
    async def StreamLatestMessages(self, stream: 'grpclib.server.Stream[gossiping_pb2.StreamLatestMessagesRequest, consensus_pb2.Block.Justification]') -> None:
        pass

    @abc.abstractmethod
    async def StreamBlockSummaries(self, stream: 'grpclib.server.Stream[gossiping_pb2.StreamBlockSummariesRequest, consensus_pb2.BlockSummary]') -> None:
        pass

    @abc.abstractmethod
    async def StreamDeploySummaries(self, stream: 'grpclib.server.Stream[gossiping_pb2.StreamDeploySummariesRequest, consensus_pb2.DeploySummary]') -> None:
        pass

    @abc.abstractmethod
    async def GetBlockChunked(self, stream: 'grpclib.server.Stream[gossiping_pb2.GetBlockChunkedRequest, gossiping_pb2.Chunk]') -> None:
        pass

    @abc.abstractmethod
    async def StreamDeploysChunked(self, stream: 'grpclib.server.Stream[gossiping_pb2.StreamDeploysChunkedRequest, gossiping_pb2.Chunk]') -> None:
        pass

    @abc.abstractmethod
    async def GetGenesisCandidate(self, stream: 'grpclib.server.Stream[gossiping_pb2.GetGenesisCandidateRequest, consensus_pb2.GenesisCandidate]') -> None:
        pass

    @abc.abstractmethod
    async def AddApproval(self, stream: 'grpclib.server.Stream[gossiping_pb2.AddApprovalRequest, empty_pb2.Empty]') -> None:
        pass

    @abc.abstractmethod
    async def StreamDagSliceBlockSummaries(self, stream: 'grpclib.server.Stream[gossiping_pb2.StreamDagSliceBlockSummariesRequest, consensus_pb2.BlockSummary]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/io.casperlabs.comm.gossiping.GossipService/NewBlocks': grpclib.const.Handler(
                self.NewBlocks,
                grpclib.const.Cardinality.UNARY_UNARY,
                gossiping_pb2.NewBlocksRequest,
                gossiping_pb2.NewBlocksResponse,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/NewDeploys': grpclib.const.Handler(
                self.NewDeploys,
                grpclib.const.Cardinality.UNARY_UNARY,
                gossiping_pb2.NewDeploysRequest,
                gossiping_pb2.NewDeploysResponse,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/StreamAncestorBlockSummaries': grpclib.const.Handler(
                self.StreamAncestorBlockSummaries,
                grpclib.const.Cardinality.UNARY_STREAM,
                gossiping_pb2.StreamAncestorBlockSummariesRequest,
                consensus_pb2.BlockSummary,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/StreamLatestMessages': grpclib.const.Handler(
                self.StreamLatestMessages,
                grpclib.const.Cardinality.UNARY_STREAM,
                gossiping_pb2.StreamLatestMessagesRequest,
                consensus_pb2.Block.Justification,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/StreamBlockSummaries': grpclib.const.Handler(
                self.StreamBlockSummaries,
                grpclib.const.Cardinality.UNARY_STREAM,
                gossiping_pb2.StreamBlockSummariesRequest,
                consensus_pb2.BlockSummary,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/StreamDeploySummaries': grpclib.const.Handler(
                self.StreamDeploySummaries,
                grpclib.const.Cardinality.UNARY_STREAM,
                gossiping_pb2.StreamDeploySummariesRequest,
                consensus_pb2.DeploySummary,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/GetBlockChunked': grpclib.const.Handler(
                self.GetBlockChunked,
                grpclib.const.Cardinality.UNARY_STREAM,
                gossiping_pb2.GetBlockChunkedRequest,
                gossiping_pb2.Chunk,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/StreamDeploysChunked': grpclib.const.Handler(
                self.StreamDeploysChunked,
                grpclib.const.Cardinality.UNARY_STREAM,
                gossiping_pb2.StreamDeploysChunkedRequest,
                gossiping_pb2.Chunk,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/GetGenesisCandidate': grpclib.const.Handler(
                self.GetGenesisCandidate,
                grpclib.const.Cardinality.UNARY_UNARY,
                gossiping_pb2.GetGenesisCandidateRequest,
                consensus_pb2.GenesisCandidate,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/AddApproval': grpclib.const.Handler(
                self.AddApproval,
                grpclib.const.Cardinality.UNARY_UNARY,
                gossiping_pb2.AddApprovalRequest,
                empty_pb2.Empty,
            ),
            '/io.casperlabs.comm.gossiping.GossipService/StreamDagSliceBlockSummaries': grpclib.const.Handler(
                self.StreamDagSliceBlockSummaries,
                grpclib.const.Cardinality.UNARY_STREAM,
                gossiping_pb2.StreamDagSliceBlockSummariesRequest,
                consensus_pb2.BlockSummary,
            ),
        }


class GossipServiceStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.NewBlocks = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/NewBlocks',
            gossiping_pb2.NewBlocksRequest,
            gossiping_pb2.NewBlocksResponse,
        )
        self.NewDeploys = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/NewDeploys',
            gossiping_pb2.NewDeploysRequest,
            gossiping_pb2.NewDeploysResponse,
        )
        self.StreamAncestorBlockSummaries = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/StreamAncestorBlockSummaries',
            gossiping_pb2.StreamAncestorBlockSummariesRequest,
            consensus_pb2.BlockSummary,
        )
        self.StreamLatestMessages = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/StreamLatestMessages',
            gossiping_pb2.StreamLatestMessagesRequest,
            consensus_pb2.Block.Justification,
        )
        self.StreamBlockSummaries = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/StreamBlockSummaries',
            gossiping_pb2.StreamBlockSummariesRequest,
            consensus_pb2.BlockSummary,
        )
        self.StreamDeploySummaries = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/StreamDeploySummaries',
            gossiping_pb2.StreamDeploySummariesRequest,
            consensus_pb2.DeploySummary,
        )
        self.GetBlockChunked = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/GetBlockChunked',
            gossiping_pb2.GetBlockChunkedRequest,
            gossiping_pb2.Chunk,
        )
        self.StreamDeploysChunked = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/StreamDeploysChunked',
            gossiping_pb2.StreamDeploysChunkedRequest,
            gossiping_pb2.Chunk,
        )
        self.GetGenesisCandidate = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/GetGenesisCandidate',
            gossiping_pb2.GetGenesisCandidateRequest,
            consensus_pb2.GenesisCandidate,
        )
        self.AddApproval = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/AddApproval',
            gossiping_pb2.AddApprovalRequest,
            empty_pb2.Empty,
        )
        self.StreamDagSliceBlockSummaries = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.comm.gossiping.GossipService/StreamDagSliceBlockSummaries',
            gossiping_pb2.StreamDagSliceBlockSummariesRequest,
            consensus_pb2.BlockSummary,
        )



# -*- coding: utf-8 -*-
from typing import Dict

from casperlabs_client import CasperLabsClient
from casperlabs_client.decorators import guarded_command

NAME: str = "balance"
HELP: str = "Returns the balance of the account at the specified block."
OPTIONS = [
    [("-a", "--address"), dict(required=True, type=str, help="Account's hash in hex.")],
    [
        ("-b", "--block-hash"),
        dict(required=True, type=str, help="Hash of the block to query the state of"),
    ],
]


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    response = casperlabs_client.balance(args.get("address"), args.get("block_hash"))
    print(response)



# -*- coding: utf-8 -*-
import base64
from abc import ABC, abstractmethod
from typing import Union
from pathlib import Path

from casperlabs_client import crypto, consts, io


class KeyHolder(ABC):
    """ Abstract class for loading, generating and handling public/private key holders """

    def __init__(
        self,
        private_key_pem: bytes = None,
        private_key=None,
        public_key_pem: bytes = None,
        public_key=None,
        algorithm: str = None,
    ):
        if not any((private_key, private_key_pem, public_key, public_key_pem)):
            raise ValueError("No public or private key information provided.")
        self._private_key_pem = private_key_pem
        self._private_key = private_key
        self._public_key_pem = public_key_pem
        self._public_key = public_key
        self._algorithm = algorithm

    @property
    def algorithm(self):
        """ String representation of the key algorithm """
        return self._algorithm

    @property
    def private_key_pem(self) -> bytes:
        """ Returns or generates private key pem data from other internal fields """
        if self._private_key_pem is None:
            if self._private_key is None:
                raise ValueError("Must have either _private_key or _private_key_pem.")
            self._private_key_pem = self._private_key_pem_from_private_key()
        return self._private_key_pem

    @abstractmethod
    def _private_key_pem_from_private_key(self) -> bytes:
        pass

    @property
    def private_key(self) -> bytes:
        """ Returns or generates private key bytes from other internal fields """
        if self._private_key is None:
            if self._private_key_pem is None:
                raise ValueError("Must have either _private_key or _private_key_pem.")
            self._private_key = self._private_key_from_private_key_pem()
        return self._private_key

    @property
    def private_key_hex(self) -> str:
        """ Returns or generates private key in hex encoding from other internal fields """
        return self.private_key.hex()

    @property
    def private_key_base64(self) -> str:
        """ Returns or generates private key in base64 encoding from other internal fields """
        return base64.b64encode(self.private_key).decode("UTF-8")

    @abstractmethod
    def _private_key_from_private_key_pem(self) -> bytes:
        pass

    @property
    def public_key_pem(self) -> bytes:
        """ Returns or generates public key pem data from other internal fields """
        if self._public_key_pem is None:
            self._public_key_pem = self._public_key_pem_from_public_key()
        return self._public_key_pem

    @abstractmethod
    def _public_key_pem_from_public_key(self) -> bytes:
        pass

    @property
    def public_key(self) -> bytes:
        """ Returns or generates public key bytes from other internal fields """
        if self._public_key is None:
            if self._public_key_pem:
                self._public_key = self._public_key_from_public_key_pem()
            elif self.private_key:
                self._public_key = self._public_key_from_private_key()
            else:
                raise ValueError("No values given to derive public key")
        return self._public_key

    @property
    def public_key_hex(self) -> str:
        """ Returns or generates public key in hex encoding from other internal fields """
        return self.public_key.hex()

    @property
    def public_key_base64(self) -> str:
        """ Returns or generates public key in base64 encoding from other internal fields """
        return base64.b64encode(self.public_key).decode("UTF-8")

    @abstractmethod
    def _public_key_from_public_key_pem(self):
        pass

    @abstractmethod
    def _public_key_from_private_key(self):
        pass

    def save_pem_files(
        self,
        save_directory: Union[str, Path],
        filename_prefix: str = consts.DEFAULT_KEY_FILENAME_PREFIX,
    ) -> None:
        """
        Save key pairs out as public and private pem files.

        :param save_directory:   str or Path to directory for saving pem files.
        :param filename_prefix:  prefix of filename to be used for save.
        """
        private_path = (
            Path(save_directory)
            / f"{filename_prefix}{consts.PRIVATE_KEY_FILENAME_SUFFIX}"
        )
        io.write_binary_file(private_path, self.private_key_pem)

        public_path = (
            Path(save_directory)
            / f"{filename_prefix}{consts.PUBLIC_KEY_FILENAME_SUFFIX}"
        )
        io.write_binary_file(public_path, self.public_key_pem)

    def save_hex_base64_files(
        self,
        directory: Union[Path, str],
        filename_prefix: str = consts.DEFAULT_KEY_FILENAME_PREFIX,
    ) -> None:
        """
        Saves out hex/base16 and base64 versions of account hash and public key.

        :param directory: Directory in which to create files. Must exist.
        :param filename_prefix: Prefix to use in files. Defaults to consts.DEFAULT_KEY_FILENAME_PREFIX

        Generated files:
            {filename_prefix}-id-hex  # account hash in hex/base16 format.
            {filename_prefix}-id      # account hash in base64 format.
            {filename_prefix}-pk-hex  # account public key in hex/base16 format.
            {filename_prefix}-pk      # account public key in base64 format.
        """
        pairs = [
            (consts.ACCOUNT_PRIVATE_KEY_HEX_FILENAME_SUFFIX, self.public_key_hex),
            (
                consts.ACCOUNT_PRIVATE_KEY_BASE64_FILENAME_SUFFIX,
                self.private_key_base64,
            ),
            (consts.ACCOUNT_HASH_HEX_FILENAME_SUFFIX, self.account_hash_hex),
            (consts.ACCOUNT_HASH_BASE64_FILENAME_SUFFIX, self.account_hash_base64),
        ]
        for suffix, value in pairs:
            io.write_file(directory / f"{filename_prefix}{suffix}", value)

    @staticmethod
    @abstractmethod
    def from_private_key_path(private_key_pem_path: Union[str, Path]) -> "KeyHolder":
        pass

    @staticmethod
    @abstractmethod
    def from_public_key_path(public_key_pem_path: Union[str, Path]) -> "KeyHolder":
        pass

    @abstractmethod
    def sign(self, data: bytes) -> bytes:
        pass

    @property
    def account_hash(self) -> bytes:
        """ Generate hash of public key and key algorithm for use as primary identifier in the system as bytes """
        # account hash is the one place where algorithm is used in upper case.
        return crypto.blake2b_hash(
            self.algorithm.upper().encode("UTF-8") + b"\x00" + self.public_key
        )

    @property
    def account_hash_hex(self) -> str:
        """ Generate hash of public key and key algorithm for use as primary identifier in the system as hex str """
        return self.account_hash.hex()

    @property
    def account_hash_base64(self) -> str:
        """ Generate hash of public key and key algorithm for use as primary identifier in the system as base64 str """
        return base64.b64encode(self.account_hash).decode("UTF-8")



# -*- coding: utf-8 -*-
import functools
import logging
import sys
import time

import grpc
from grpc._channel import _Rendezvous

NUMBER_OF_RETRIES = 5
# Initial delay in seconds before an attempt to retry
INITIAL_DELAY = 0.3


def retry_wrapper(function, *args):
    delay = INITIAL_DELAY
    for i in range(NUMBER_OF_RETRIES):
        try:
            return function(*args)
        except _Rendezvous as e:
            if e.code() == grpc.StatusCode.UNAVAILABLE and i < NUMBER_OF_RETRIES - 1:
                # TODO: Look at using backoff PyPi package.
                delay += delay
                logging.warning(f"Retrying after {e} in {delay} seconds")
                time.sleep(delay)
            else:
                raise


def retry_unary(function):
    @functools.wraps(function)
    def wrapper(*args):
        return retry_wrapper(function, *args)

    return wrapper


def retry_stream(function):
    @functools.wraps(function)
    def wrapper(*args):
        yield from retry_wrapper(function, *args)

    return wrapper


def guarded_command(function):
    """
    Decorator of functions that implement CLI commands.

    Occasionally the node can throw some exceptions instead of properly sending us a response,
    those will be deserialized on our end and rethrown by the gRPC layer.
    In this case we want to catch the exception and return a non-zero return code to the shell.

    :param function:  function to be decorated
    :return:
    """

    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        try:
            rc = function(*args, **kwargs)
            # Generally the CLI commands are assumed to succeed if they don't throw,
            # but they can also return a positive error code if they need to.
            if rc is not None:
                return rc
            return 0
        except Exception as e:
            print(str(e), file=sys.stderr)
            return 1

    return wrapper



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: ipc.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

import state_pb2
import transforms_pb2
import ipc_pb2


class ExecutionEngineServiceBase(abc.ABC):

    @abc.abstractmethod
    async def commit(self, stream: 'grpclib.server.Stream[ipc_pb2.CommitRequest, ipc_pb2.CommitResponse]') -> None:
        pass

    @abc.abstractmethod
    async def query(self, stream: 'grpclib.server.Stream[ipc_pb2.QueryRequest, ipc_pb2.QueryResponse]') -> None:
        pass

    @abc.abstractmethod
    async def execute(self, stream: 'grpclib.server.Stream[ipc_pb2.ExecuteRequest, ipc_pb2.ExecuteResponse]') -> None:
        pass

    @abc.abstractmethod
    async def run_genesis(self, stream: 'grpclib.server.Stream[ipc_pb2.RunGenesisRequest, ipc_pb2.GenesisResponse]') -> None:
        pass

    @abc.abstractmethod
    async def upgrade(self, stream: 'grpclib.server.Stream[ipc_pb2.UpgradeRequest, ipc_pb2.UpgradeResponse]') -> None:
        pass

    @abc.abstractmethod
    async def bid_state(self, stream: 'grpclib.server.Stream[ipc_pb2.BidStateRequest, ipc_pb2.BidStateResponse]') -> None:
        pass

    @abc.abstractmethod
    async def distribute_rewards(self, stream: 'grpclib.server.Stream[ipc_pb2.DistributeRewardsRequest, ipc_pb2.DistributeRewardsResponse]') -> None:
        pass

    @abc.abstractmethod
    async def slash(self, stream: 'grpclib.server.Stream[ipc_pb2.SlashRequest, ipc_pb2.SlashResponse]') -> None:
        pass

    @abc.abstractmethod
    async def unbond_payout(self, stream: 'grpclib.server.Stream[ipc_pb2.UnbondPayoutRequest, ipc_pb2.UnbondPayoutResponse]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/io.casperlabs.ipc.ExecutionEngineService/commit': grpclib.const.Handler(
                self.commit,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.CommitRequest,
                ipc_pb2.CommitResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/query': grpclib.const.Handler(
                self.query,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.QueryRequest,
                ipc_pb2.QueryResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/execute': grpclib.const.Handler(
                self.execute,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.ExecuteRequest,
                ipc_pb2.ExecuteResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/run_genesis': grpclib.const.Handler(
                self.run_genesis,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.RunGenesisRequest,
                ipc_pb2.GenesisResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/upgrade': grpclib.const.Handler(
                self.upgrade,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.UpgradeRequest,
                ipc_pb2.UpgradeResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/bid_state': grpclib.const.Handler(
                self.bid_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.BidStateRequest,
                ipc_pb2.BidStateResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/distribute_rewards': grpclib.const.Handler(
                self.distribute_rewards,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.DistributeRewardsRequest,
                ipc_pb2.DistributeRewardsResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/slash': grpclib.const.Handler(
                self.slash,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.SlashRequest,
                ipc_pb2.SlashResponse,
            ),
            '/io.casperlabs.ipc.ExecutionEngineService/unbond_payout': grpclib.const.Handler(
                self.unbond_payout,
                grpclib.const.Cardinality.UNARY_UNARY,
                ipc_pb2.UnbondPayoutRequest,
                ipc_pb2.UnbondPayoutResponse,
            ),
        }


class ExecutionEngineServiceStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.commit = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/commit',
            ipc_pb2.CommitRequest,
            ipc_pb2.CommitResponse,
        )
        self.query = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/query',
            ipc_pb2.QueryRequest,
            ipc_pb2.QueryResponse,
        )
        self.execute = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/execute',
            ipc_pb2.ExecuteRequest,
            ipc_pb2.ExecuteResponse,
        )
        self.run_genesis = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/run_genesis',
            ipc_pb2.RunGenesisRequest,
            ipc_pb2.GenesisResponse,
        )
        self.upgrade = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/upgrade',
            ipc_pb2.UpgradeRequest,
            ipc_pb2.UpgradeResponse,
        )
        self.bid_state = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/bid_state',
            ipc_pb2.BidStateRequest,
            ipc_pb2.BidStateResponse,
        )
        self.distribute_rewards = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/distribute_rewards',
            ipc_pb2.DistributeRewardsRequest,
            ipc_pb2.DistributeRewardsResponse,
        )
        self.slash = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/slash',
            ipc_pb2.SlashRequest,
            ipc_pb2.SlashResponse,
        )
        self.unbond_payout = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.ipc.ExecutionEngineService/unbond_payout',
            ipc_pb2.UnbondPayoutRequest,
            ipc_pb2.UnbondPayoutResponse,
        )



# -*- coding: utf-8 -*-
import sys
from typing import Dict

from casperlabs_client import consensus_pb2 as consensus, io, CasperLabsClient
from casperlabs_client.commands.common_options import (
    private_key_option,
    ALGORITHM_OPTION,
)

from casperlabs_client.decorators import guarded_command

NAME: str = "sign-deploy"
HELP: str = "Cryptographically signs a deploy. The signature is appended to existing approvals."
OPTIONS = [
    [
        ("-o", "--signed-deploy-path"),
        dict(
            required=False,
            default=None,
            help=(
                "Path to the file where signed deploy will be saved. "
                "Optional, if not provided the deploy will be printed to STDOUT."
            ),
        ),
    ],
    [
        ("-i", "--deploy-path"),
        dict(required=False, default=None, help="Path to the deploy file."),
    ],
    private_key_option(required=True),
    ALGORITHM_OPTION,
]


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    private_key = args.get("private_key")
    algorithm = args.get("algorithm")
    deploy_path = args.get("deploy_path")
    signed_deploy_path = args.get("signed_deploy_path")
    deploy = None
    if not deploy_path:
        deploy = consensus.Deploy()
        deploy.ParseFromString(sys.stdin.read())

    signed_deploy = casperlabs_client.sign_deploy(
        private_key_pem_file=private_key,
        algorithm=algorithm,
        deploy=deploy,
        deploy_file=deploy_path,
    )
    serialized_deploy = signed_deploy.SerializeToString()
    if signed_deploy_path is None:
        sys.stdout.write(serialized_deploy)
    else:
        io.write_binary_file(signed_deploy_path, serialized_deploy)



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: kademlia.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

import node_pb2
import kademlia_pb2


class KademliaServiceBase(abc.ABC):

    @abc.abstractmethod
    async def Ping(self, stream: 'grpclib.server.Stream[kademlia_pb2.PingRequest, kademlia_pb2.PingResponse]') -> None:
        pass

    @abc.abstractmethod
    async def Lookup(self, stream: 'grpclib.server.Stream[kademlia_pb2.LookupRequest, kademlia_pb2.LookupResponse]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/io.casperlabs.comm.discovery.KademliaService/Ping': grpclib.const.Handler(
                self.Ping,
                grpclib.const.Cardinality.UNARY_UNARY,
                kademlia_pb2.PingRequest,
                kademlia_pb2.PingResponse,
            ),
            '/io.casperlabs.comm.discovery.KademliaService/Lookup': grpclib.const.Handler(
                self.Lookup,
                grpclib.const.Cardinality.UNARY_UNARY,
                kademlia_pb2.LookupRequest,
                kademlia_pb2.LookupResponse,
            ),
        }


class KademliaServiceStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.Ping = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.comm.discovery.KademliaService/Ping',
            kademlia_pb2.PingRequest,
            kademlia_pb2.PingResponse,
        )
        self.Lookup = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.comm.discovery.KademliaService/Lookup',
            kademlia_pb2.LookupRequest,
            kademlia_pb2.LookupResponse,
        )



# -*- coding: utf-8 -*-
import argparse
import os
from pathlib import Path

from casperlabs_client.consts import SUPPORTED_KEY_ALGORITHMS


def algorithm(algorithm_name):
    """ Check the algorithm string is supported """
    if algorithm_name in SUPPORTED_KEY_ALGORITHMS:
        return algorithm_name
    raise argparse.ArgumentTypeError(
        f"{algorithm_name} is not valid. Must be in {SUPPORTED_KEY_ALGORITHMS}"
    )


def positive_integer(number):
    """Check number is an integer greater than 0"""
    n = int(number)
    if n < 1:
        raise argparse.ArgumentTypeError(f"{number} is not a positive int value")
    return n


def directory_for_write(path):
    """ Check directory exists and is writable. """
    if not os.path.exists(path):
        raise argparse.ArgumentTypeError(f"Directory '{path}' does not exist")
    if not os.path.isdir(path):
        raise argparse.ArgumentTypeError(f"'{path}' is not a directory")
    if not os.access(path, os.W_OK):
        raise argparse.ArgumentTypeError(f"'{path}' does not have writing permissions")
    return Path(path)
